var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw new Error('Dynamic require of "' + x + '" is not supported');
});

// src/ts.ts
import path2 from "path";
import ts from "typescript";

// src/transform/index.ts
import path from "path";

// src/transform/vue.ts
function transformForVue(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = __require("vue/compiler-sfc");
  const result = compiler.parse(code);
  const compiled = compiler.compileScript(result.descriptor, {
    id: "id",
    reactivityTransform: true
  });
  return compiled.content;
}

// src/transform/svelte.ts
function transformForSvelte(code, context) {
  if (context.current) {
    return code;
  }
  const svelte2tsx = __require("svelte2tsx");
  const result = svelte2tsx.svelte2tsx(code, {
    filename: context.filePath
  });
  return `/// <reference types="svelte2tsx/svelte-shims" />
  
${result.code}`;
}

// src/transform/astro.ts
function transformForAstro(code, context) {
  if (context.current) {
    return code;
  }
  const compiler = __require("astrojs-compiler-sync");
  const result = compiler.convertToTSX(code, {
    sourcefile: context.filePath
  });
  return result.code;
}

// src/transform/index.ts
function transformExtraFile(code, context) {
  const ext = path.extname(context.filePath);
  const transform = ext === ".vue" ? transformForVue : ext === ".svelte" ? transformForSvelte : ext === ".astro" ? transformForAstro : () => code;
  try {
    return transform(code, context);
  } catch (e) {
  }
  return code;
}

// src/ts.ts
var TSServiceManager = class {
  constructor() {
    this.tsServices = /* @__PURE__ */ new Map();
  }
  getProgram(code, options) {
    const tsconfigPath = options.project;
    const extraFileExtensions = [...new Set(options.extraFileExtensions)];
    let serviceList = this.tsServices.get(tsconfigPath);
    if (!serviceList) {
      serviceList = [];
      this.tsServices.set(tsconfigPath, serviceList);
    }
    let service = serviceList.find(
      (service2) => extraFileExtensions.every(
        (ext) => service2.extraFileExtensions.includes(ext)
      )
    );
    if (!service) {
      service = new TSService(tsconfigPath, extraFileExtensions);
      serviceList.unshift(service);
    }
    return service.getProgram(code, options.filePath);
  }
};
var TSService = class {
  constructor(tsconfigPath, extraFileExtensions) {
    this.patchedHostSet = /* @__PURE__ */ new WeakSet();
    this.currTarget = {
      code: "",
      filePath: "",
      dirMap: /* @__PURE__ */ new Map()
    };
    this.fileWatchCallbacks = /* @__PURE__ */ new Map();
    this.tsconfigPath = tsconfigPath;
    this.extraFileExtensions = extraFileExtensions;
    this.watch = this.createWatch(tsconfigPath, extraFileExtensions);
  }
  getProgram(code, filePath) {
    var _a;
    const normalized = normalizeFileName(
      toRealFileName(filePath, this.extraFileExtensions)
    );
    const lastTarget = this.currTarget;
    const dirMap = /* @__PURE__ */ new Map();
    let childPath = normalized;
    for (const dirName of iterateDirs(normalized)) {
      dirMap.set(dirName, { path: childPath, name: path2.basename(childPath) });
      childPath = dirName;
    }
    this.currTarget = {
      code,
      filePath: normalized,
      dirMap
    };
    for (const { filePath: targetPath } of [this.currTarget, lastTarget]) {
      if (!targetPath)
        continue;
      if (!ts.sys.fileExists(targetPath)) {
        (_a = this.fileWatchCallbacks.get(normalizeFileName(this.tsconfigPath))) == null ? void 0 : _a.update();
      }
    }
    getRefreshTargetFileNames(
      lastTarget.filePath,
      this.extraFileExtensions
    ).forEach((vFilePath) => {
      var _a2;
      (_a2 = this.fileWatchCallbacks.get(vFilePath)) == null ? void 0 : _a2.resetTarget();
    });
    getRefreshTargetFileNames(
      this.currTarget.filePath,
      this.extraFileExtensions
    ).forEach((vFilePath) => {
      var _a2;
      (_a2 = this.fileWatchCallbacks.get(vFilePath)) == null ? void 0 : _a2.setupTarget();
    });
    const program = this.watch.getProgram().getProgram();
    program.getTypeChecker();
    return program;
  }
  createWatch(tsconfigPath, extraFileExtensions) {
    const createAbstractBuilder = (...args) => {
      const [
        rootNames,
        options,
        argHost,
        oldProgram,
        configFileParsingDiagnostics,
        projectReferences
      ] = args;
      const host = argHost;
      if (!this.patchedHostSet.has(host)) {
        this.patchedHostSet.add(host);
        const getTargetSourceFile = (fileName, languageVersionOrOptions) => {
          var _a, _b;
          if (this.currTarget.filePath === normalizeFileName(fileName) && isExtra(fileName, extraFileExtensions)) {
            return (_b = (_a = this.currTarget).sourceFile) != null ? _b : _a.sourceFile = ts.createSourceFile(
              this.currTarget.filePath,
              this.currTarget.code,
              languageVersionOrOptions,
              true,
              ts.ScriptKind.TSX
            );
          }
          return null;
        };
        const original2 = {
          getSourceFile: host.getSourceFile,
          getSourceFileByPath: host.getSourceFileByPath
        };
        host.getSourceFile = (fileName, languageVersionOrOptions, ...args2) => {
          var _a;
          const originalSourceFile = original2.getSourceFile.call(
            host,
            fileName,
            languageVersionOrOptions,
            ...args2
          );
          return (_a = getTargetSourceFile(fileName, languageVersionOrOptions)) != null ? _a : originalSourceFile;
        };
        host.getSourceFileByPath = (fileName, path4, languageVersionOrOptions, ...args2) => {
          var _a;
          const originalSourceFile = original2.getSourceFileByPath.call(
            host,
            fileName,
            path4,
            languageVersionOrOptions,
            ...args2
          );
          return (_a = getTargetSourceFile(fileName, languageVersionOrOptions)) != null ? _a : originalSourceFile;
        };
      }
      return ts.createAbstractBuilder(
        rootNames,
        options,
        host,
        oldProgram,
        configFileParsingDiagnostics,
        projectReferences
      );
    };
    const watchCompilerHost = ts.createWatchCompilerHost(
      tsconfigPath,
      {
        noEmit: true,
        jsx: ts.JsxEmit.Preserve,
        // This option is required if `includes` only includes `*.vue` files.
        // However, the option is not in the documentation.
        // https://github.com/microsoft/TypeScript/issues/28447
        allowNonTsExtensions: true
      },
      ts.sys,
      createAbstractBuilder,
      (diagnostic) => {
        throw new Error(formatDiagnostics([diagnostic]));
      },
      () => {
      },
      void 0,
      extraFileExtensions.map((extension) => ({
        extension,
        isMixedContent: true,
        scriptKind: ts.ScriptKind.Deferred
      }))
    );
    const original = {
      // eslint-disable-next-line @typescript-eslint/unbound-method -- Store original
      readFile: watchCompilerHost.readFile,
      // eslint-disable-next-line @typescript-eslint/unbound-method -- Store original
      fileExists: watchCompilerHost.fileExists,
      // eslint-disable-next-line @typescript-eslint/unbound-method -- Store original
      readDirectory: watchCompilerHost.readDirectory,
      // eslint-disable-next-line @typescript-eslint/unbound-method -- Store original
      directoryExists: watchCompilerHost.directoryExists,
      // eslint-disable-next-line @typescript-eslint/unbound-method -- Store original
      getDirectories: watchCompilerHost.getDirectories
    };
    watchCompilerHost.getDirectories = (dirName, ...args) => {
      var _a;
      const result = distinctArray(
        ...original.getDirectories.call(watchCompilerHost, dirName, ...args),
        // Include the path to the target file if the target file does not actually exist.
        (_a = this.currTarget.dirMap.get(normalizeFileName(dirName))) == null ? void 0 : _a.name
      );
      return result;
    };
    watchCompilerHost.directoryExists = (dirName, ...args) => {
      return original.directoryExists.call(watchCompilerHost, dirName, ...args) || // Include the path to the target file if the target file does not actually exist.
      this.currTarget.dirMap.has(normalizeFileName(dirName));
    };
    watchCompilerHost.readDirectory = (dirName, ...args) => {
      let results = original.readDirectory.call(
        watchCompilerHost,
        dirName,
        ...args
      );
      const file = this.currTarget.dirMap.get(normalizeFileName(dirName));
      if (file) {
        if (file.path === this.currTarget.filePath) {
          results.push(file.path);
        } else {
          results = results.filter((f) => file.path !== f && file.name !== f);
        }
      }
      return distinctArray(...results);
    };
    watchCompilerHost.readFile = (fileName, ...args) => {
      const realFileName = toRealFileName(fileName, extraFileExtensions);
      const normalized = normalizeFileName(realFileName);
      if (this.currTarget.filePath === normalized) {
        return transformExtraFile(this.currTarget.code, {
          filePath: normalized,
          current: true
        });
      }
      if (isExtraDts(fileName, extraFileExtensions)) {
        const real = normalizeFileName(
          extraDtsToExtra(fileName, extraFileExtensions)
        );
        if (this.currTarget.filePath === real) {
          return void 0;
        }
      }
      if (isVirtualTSX(fileName, extraFileExtensions)) {
        const dts = toExtraDtsFileName(normalized, extraFileExtensions);
        if (original.fileExists.call(watchCompilerHost, dts)) {
          return void 0;
        }
      }
      const code = original.readFile.call(
        watchCompilerHost,
        realFileName,
        ...args
      );
      if (!code) {
        return code;
      }
      return transformExtraFile(code, {
        filePath: normalized,
        current: false
      });
    };
    watchCompilerHost.fileExists = (fileName, ...args) => {
      const normalizedFileName = normalizeFileName(fileName);
      if (this.currTarget.dirMap.has(normalizedFileName)) {
        return false;
      }
      const normalizedRealFileName = toRealFileName(
        normalizedFileName,
        extraFileExtensions
      );
      if (this.currTarget.filePath === normalizedRealFileName) {
        return true;
      }
      if (original.fileExists.call(
        watchCompilerHost,
        normalizedRealFileName,
        ...args
      )) {
        if (isVirtualTSX(fileName, extraFileExtensions)) {
          if (original.fileExists.call(
            watchCompilerHost,
            toExtraDtsFileName(normalizedRealFileName, extraFileExtensions),
            ...args
          )) {
            return false;
          }
        }
        return true;
      }
      return false;
    };
    watchCompilerHost.watchFile = (fileName, callback) => {
      const normalized = normalizeFileName(fileName);
      this.fileWatchCallbacks.set(normalized, {
        // The function is called when the file is targeted for parsing.
        setupTarget: () => {
          if (isExtraDts(fileName, extraFileExtensions)) {
            callback(fileName, ts.FileWatcherEventKind.Deleted);
          } else if (isVirtualTSX(fileName, extraFileExtensions)) {
            callback(fileName, ts.FileWatcherEventKind.Created);
          } else {
            callback(fileName, ts.FileWatcherEventKind.Changed);
          }
        },
        // The function is called when the file leaves the target of parsing.
        resetTarget: () => {
          if (isExtraDts(fileName, extraFileExtensions)) {
            callback(fileName, ts.FileWatcherEventKind.Created);
          } else if (isVirtualTSX(fileName, extraFileExtensions)) {
            callback(fileName, ts.FileWatcherEventKind.Deleted);
          } else {
            callback(fileName, ts.FileWatcherEventKind.Changed);
          }
        },
        update: () => callback(fileName, ts.FileWatcherEventKind.Changed)
      });
      return {
        close: () => {
          this.fileWatchCallbacks.delete(normalized);
        }
      };
    };
    watchCompilerHost.watchDirectory = () => {
      return {
        close: () => {
        }
      };
    };
    watchCompilerHost.afterProgramCreate = (program) => {
      const originalDiagnostics = program.getConfigFileParsingDiagnostics();
      const configFileDiagnostics = originalDiagnostics.filter(
        (diag) => diag.category === ts.DiagnosticCategory.Error && diag.code !== 18003
      );
      if (configFileDiagnostics.length > 0) {
        throw new Error(formatDiagnostics(configFileDiagnostics));
      }
    };
    const watch = ts.createWatchProgram(watchCompilerHost);
    return watch;
  }
};
function getRefreshTargetFileNames(fileName, extraFileExtensions) {
  if (isExtra(fileName, extraFileExtensions)) {
    return [`${fileName}.tsx`, `${fileName}.d.ts`, fileName];
  }
  return [fileName];
}
function toExtraDtsFileName(fileName, extraFileExtensions) {
  if (isExtra(fileName, extraFileExtensions)) {
    return `${fileName}.d.ts`;
  }
  return fileName;
}
function toRealFileName(fileName, extraFileExtensions) {
  if (isVirtualTSX(fileName, extraFileExtensions)) {
    return fileName.slice(0, -4);
  }
  return fileName;
}
function extraDtsToExtra(fileName, extraFileExtensions) {
  if (isExtraDts(fileName, extraFileExtensions)) {
    return fileName.slice(0, -5);
  }
  return fileName;
}
function isExtra(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(extraFileExtension)) {
      return true;
    }
  }
  return false;
}
function isVirtualTSX(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(`${extraFileExtension}.tsx`)) {
      return true;
    }
  }
  return false;
}
function isExtraDts(fileName, extraFileExtensions) {
  for (const extraFileExtension of extraFileExtensions) {
    if (fileName.endsWith(`${extraFileExtension}.d.ts`)) {
      return true;
    }
  }
  return false;
}
function formatDiagnostics(diagnostics) {
  return ts.formatDiagnostics(diagnostics, {
    getCanonicalFileName: (f) => f,
    getCurrentDirectory: () => process.cwd(),
    getNewLine: () => "\n"
  });
}
function normalizeFileName(fileName) {
  let normalized = path2.normalize(fileName);
  if (normalized.endsWith(path2.sep)) {
    normalized = normalized.slice(0, -1);
  }
  if (ts.sys.useCaseSensitiveFileNames) {
    return toAbsolutePath(normalized, null);
  }
  return toAbsolutePath(normalized.toLowerCase(), null);
}
function toAbsolutePath(filePath, baseDir) {
  return path2.isAbsolute(filePath) ? filePath : path2.join(baseDir || process.cwd(), filePath);
}
function* iterateDirs(filePath) {
  let target = filePath;
  let parent;
  while ((parent = path2.dirname(target)) !== target) {
    yield parent;
    target = parent;
  }
}
function distinctArray(...list) {
  return [
    ...new Set(
      ts.sys.useCaseSensitiveFileNames ? list : list.map((s) => s == null ? void 0 : s.toLowerCase())
    )
  ].filter((s) => s != null);
}

// src/index.ts
import * as tsEslintParser from "@typescript-eslint/parser";

// src/utils/get-project-config-files.ts
import fs from "fs";
import path3 from "path";
function getProjectConfigFiles(options) {
  const tsconfigRootDir = typeof options.tsconfigRootDir === "string" ? options.tsconfigRootDir : process.cwd();
  if (options.project !== true) {
    return Array.isArray(options.project) ? options.project : [options.project];
  }
  let directory = path3.dirname(options.filePath);
  const checkedDirectories = [directory];
  do {
    const tsconfigPath = path3.join(directory, "tsconfig.json");
    if (fs.existsSync(tsconfigPath)) {
      return [tsconfigPath];
    }
    directory = path3.dirname(directory);
    checkedDirectories.push(directory);
  } while (directory.length > 1 && directory.length >= tsconfigRootDir.length);
  throw new Error(
    `project was set to \`true\` but couldn't find any tsconfig.json relative to '${options.filePath}' within '${tsconfigRootDir}'.`
  );
}

// src/index.ts
var DEFAULT_EXTRA_FILE_EXTENSIONS = [".vue", ".svelte", ".astro"];
var tsServiceManager = new TSServiceManager();
var name = "typescript-eslint-parser-for-extra-files";
function parseForESLint2(code, options = {}) {
  if (!options.project) {
    return tsEslintParser.parseForESLint(code, options);
  }
  const extraFileExtensions = options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS;
  const programs = [];
  for (const option of iterateOptions(options)) {
    programs.push(tsServiceManager.getProgram(code, option));
  }
  const filePath = options.filePath;
  const parserOptions = __spreadProps(__spreadValues({}, options), {
    filePath,
    programs,
    extraFileExtensions
  });
  return tsEslintParser.parseForESLint(code, parserOptions);
}
function* iterateOptions(options) {
  if (!options) {
    throw new Error("`parserOptions` is required.");
  }
  if (!options.filePath) {
    throw new Error("`filePath` is required.");
  }
  if (!options.project) {
    throw new Error(
      "Specify `parserOptions.project`. Otherwise there is no point in using this parser."
    );
  }
  for (const project of getProjectConfigFiles(options)) {
    yield {
      project,
      filePath: options.filePath,
      extraFileExtensions: options.extraFileExtensions || DEFAULT_EXTRA_FILE_EXTENSIONS
    };
  }
}
export {
  name,
  parseForESLint2 as parseForESLint
};
